/// <reference types="cheerio" />
/**
 * Base scraper class.
 *
 * node-scrap provides a base Scraper class that you will need to extend in order to build your own scraper. This class
 * takes care of the "low level" details of scraping and it lets you focus on the important stuff: scraping the data
 * you need.
 *
 * The way this works is really simple. The scraping loop consists of the following steps:
 *
 * 1. Fetch the URL be scraped using `getNextUrl()`.
 * 2. node-scrap downloads the HTML document and parses the DOM using `cheerio`.
 * 3. The `scrap($)` method takes the DOM instance and extracts the desired data.
 * 4. The `process(data)` method takes the data returned by the `scrap()` method and process it.
 * 5. The `afterEach()` method is called.
 *
 * This cycle repeats until you call the `stop()` method.
 */
declare class Scraper {
    isStopped: boolean;
    state: Record<string, any>;
    constructor();
    /**
     * Provides the default values for scraper state.
     *
     * Use the `state` property to store metadata that you want to share across methods.
     *
     * @returns the initial state for the state.
     */
    getInitialParams(): Promise<Record<string, any>>;
    /**
     * Generates the next URL to be scraped.
     *
     * Use this method to encapsulate all the pagination logic or whatever processing you need to do in order to generate
     * the next URL to be scraped.
     *
     * @returns the URL of the next page to be scrapped.
     */
    getNextUrl(): Promise<string>;
    /**
     * Scraps the page and extracts the data.
     *
     * This is where the actual data scraping takes place. Use the cheerio
     * instance to find DOM elements and extract the desired data from them. Once
     * you are done, return the scraped data.
     *
     * @param $ - cheerio instance at the root of the page.
     * @returns the scraped data.
     */
    extract($: cheerio.Root): Promise<any>;
    /**
     * Processed the scraped data.
     *
     * If you need to preprocess the scraped, store it in a database, or do
     * anything to it, this is where you do it. This method takes the return value
     * from the `extract()` method.
     *
     * @param data - the scraped data.
     */
    process(data: any): Promise<void>;
    /**
     * This method runs after `process()` in the scraping loop.
     *
     * If you need to update the `currentPage` number in the `state` property, this is where you do need to do it.
     */
    afterEach(): Promise<void>;
    /**
     * Stops the scraping loop.
     *
     * If you got to the last page and you're done scraping the website, call this method to stop.
     */
    stop(): void;
    /**
     * This method implements the logic for a `tick` in the scraping loop.
     */
    step(): Promise<void>;
    /**
     * Starts the scraping loop.
     */
    run(): Promise<void>;
}
export default Scraper;
