"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const cheerio_1 = __importDefault(require("cheerio"));
/**
 * Base scraper class.
 *
 * node-scrap provides a base Scraper class that you will need to extend in order to build your own scraper. This class
 * takes care of the "low level" details of scraping and it lets you focus on the important stuff: scraping the data
 * you need.
 *
 * The way this works is really simple. The scraping loop consists of the following steps:
 *
 * 1. Fetch the URL be scraped using `getNextUrl()`.
 * 2. node-scrap downloads the HTML document and parses the DOM using `cheerio`.
 * 3. The `scrap($)` method takes the DOM instance and extracts the desired data.
 * 4. The `process(data)` method takes the data returned by the `scrap()` method and process it.
 * 5. The `afterEach()` method is called.
 *
 * This cycle repeats until you call the `stop()` method.
 */
class Scraper {
    constructor() {
        this.isStopped = false;
        this.state = this.getInitialParams();
    }
    /**
     * Provides the default values for scraper state.
     *
     * Use the `state` property to store metadata that you want to share across methods.
     *
     * @returns the initial state for the state.
     */
    getInitialParams() {
        return __awaiter(this, void 0, void 0, function* () {
            return {};
        });
    }
    /**
     * Generates the next URL to be scraped.
     *
     * Use this method to encapsulate all the pagination logic or whatever processing you need to do in order to generate
     * the next URL to be scraped.
     *
     * @returns the URL of the next page to be scrapped.
     */
    getNextUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("Your scraper class must implement getNextUrl()");
        });
    }
    /**
     * Scraps the page and extracts the data.
     *
     * This is where the actual data scraping takes place. Use the cheerio
     * instance to find DOM elements and extract the desired data from them. Once
     * you are done, return the scraped data.
     *
     * @param $ - cheerio instance at the root of the page.
     * @returns the scraped data.
     */
    extract($) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("Your scraper class must implement the scrap() method");
        });
    }
    /**
     * Processed the scraped data.
     *
     * If you need to preprocess the scraped, store it in a database, or do
     * anything to it, this is where you do it. This method takes the return value
     * from the `extract()` method.
     *
     * @param data - the scraped data.
     */
    process(data) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log("Scraped data:", data);
        });
    }
    /**
     * This method runs after `process()` in the scraping loop.
     *
     * If you need to update the `currentPage` number in the `state` property, this is where you do need to do it.
     */
    afterEach() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * Stops the scraping loop.
     *
     * If you got to the last page and you're done scraping the website, call this method to stop.
     */
    stop() {
        this.isStopped = true;
    }
    /**
     * This method implements the logic for a `tick` in the scraping loop.
     */
    step() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = yield this.getNextUrl();
            const response = yield axios_1.default.get(url);
            const root = cheerio_1.default.load(response.data);
            const data = yield this.extract(root);
            yield this.process(data);
            yield this.afterEach();
            if (!this.isStopped) {
                this.step();
            }
        });
    }
    /**
     * Starts the scraping loop.
     */
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this.isStopped = false;
            yield this.step();
        });
    }
}
exports.default = Scraper;
